@@ -71,9 +71,6 @@ FILE_FORMAT = {
 # used for determining empty strings, which are invalid by PO spec
 EMPTY_COMMENT = "LEAVE empty space in translation"
 
-# po: new translations added through weblate use case sensitive code: pt_BR.po. Keeping them.
-LOWERCASE_EXCLUDE = [".git", ".svn", ".hg", "README.md", "po"]
-
 CONTEXT_FEMALE = "female"
 
 
@@ -106,34 +103,6 @@ def create_dir(path):
         os.makedirs(path)
 
 
-# lowercase directory
-def lowercase_rename(root_dir, items):
-    for item in items:
-        old_name = os.path.join(root_dir, item)
-        new_name = os.path.join(root_dir, item.lower())
-        if new_name != old_name:
-            print("renaming {} to {}".format(old_name, new_name))
-            os.rename(old_name, new_name)
-
-
-def lowercase_recursively(dir):  # this is the function that is actually used
-    for dir_name, subdir_list, file_list in os.walk(dir, topdown=False):
-        subdir_list[:] = [d for d in subdir_list if d not in LOWERCASE_EXCLUDE]
-        for sd in subdir_list:
-            for dname, sdir_list, file_list in os.walk(sd, topdown=False):
-                lowercase_rename(dir_name, file_list)
-                lowercase_rename(dir_name, sdir_list)
-    # why is this separate?
-    children = os.listdir(dir)
-    children[:] = [c for c in children if c not in LOWERCASE_EXCLUDE]
-    with cd(dir):
-        for c in children:
-            new_c = c.lower()
-            if c != new_c:
-                print("renaming {} to {}".format(c, new_c))
-                os.rename(c, new_c)
-
-
 def dir_or_exit(d):
     if os.path.isdir(d):
         print("Found directory {}".format(d))
@@ -234,7 +203,7 @@ def metadata(old_metadata=None, pot=False, po=False):
             "MIME-Version": "1.0",
             "Content-Type": "text/plain; charset=UTF-8",
             "Content-Transfer-Encoding": "8bit",
-            "X-Generator": "bgforge_po v.{}".format(CONFIG.version),
+            "X-Generator": "BGforge/msg2po v.{}".format(CONFIG.version),
         }
         if pot:
             data["POT-Creation-Date"] = datetime.today().strftime("%Y-%m-%d-%H:%M") + "+0000"
@@ -512,6 +481,7 @@ def file2msgstr(
     encoding=CONFIG.encoding,
     overwrite: bool = True,
     same: bool = False,
+    female_map=None,
 ):
     """returns PO file object"""
 
@@ -522,7 +492,8 @@ def file2msgstr(
     for e in po:
         for eo in e.occurrences:
             entries_dict[(eo[0], eo[1])] = e
-    female_map = female_entries(po)
+    if female_map is None:
+        female_map = female_entries(po)
 
     for t in trans.entries:
         index = t.index
@@ -534,45 +505,57 @@ def file2msgstr(
             print("WARN: no msgid found for {}:{}, skipping string\n      {}".format(occurence_path, index, value))
             continue
 
+        new_female_enties = []
         if (occurence_path, index) in entries_dict:
             # map entries to occurrences for faster access, part 2
             e: polib.POEntry = entries_dict[(occurence_path, index)]
 
-            # female entries have no occurences
-            if female_value and e.msgid in female_map:
-                fe: polib.POEntry = female_map[e.msgid]
-                if fe and (fe.msgstr != female_value):
-                    print("INFO: female translation change detected:")
-                    print("  ORIG: {}".format(e.msgid))
-                    print("  OLD:  {}".format(fe.msgstr))
-                    print("  NEW: {}".format(female_value))
-                    skip = False
-                    if not overwrite:
-                        print("  Female translation already exists, overwrite disabled, skipping")
-                        skip = True
-                    if not skip and (e.msgid == female_value):
-                        if same:
-                            print("INFO: source and female translation are the same. Using it regardless.")
-                            print("   {}".format(e.msgid))
-                            print("   {}".format(female_value))
-                        else:
-                            print(
-                                "INFO: source and female translation are the same for {}. Skipping:".format(
-                                    e.occurrences
-                                )
-                            )
-                            print("   {}".format(e.msgid))
-                            print("   {}".format(female_value))
+            if female_value:
+                # female entries have no occurences, checking if female translation already present
+                if e.msgid in female_map:
+                    fe: polib.POEntry = female_map[e.msgid]
+                    if fe and (fe.msgstr != female_value):
+                        print("INFO: female translation change detected:")
+                        print("  ORIG: {}".format(e.msgid))
+                        print("  OLD:  {}".format(fe.msgstr))
+                        print("  NEW:  {}".format(female_value))
+                        skip = False
+                        if not overwrite:
+                            print("  Female translation already exists, overwrite disabled, skipping")
                             skip = True
-                    if not skip:
-                        fe.msgstr = female_value
-                        if "fuzzy" in fe.flags:
-                            print("    Unfuzzied female entry")
-                            fe.flags.remove("fuzzy")
-                            fe.previous_msgid = None
+                        if not skip and (e.msgid == female_value):
+                            if same:
+                                print("INFO: source and female translation are the same. Using it regardless.")
+                                print("   {}".format(e.msgid))
+                                print("   {}".format(female_value))
+                            else:
+                                print(
+                                    "INFO: source and female translation are the same for {}. Skipping:".format(
+                                        e.occurrences
+                                    )
+                                )
+                                print("   {}".format(e.msgid))
+                                print("   {}".format(female_value))
+                                skip = True
+                        if not skip:
+                            fe.msgstr = female_value
+                            if "fuzzy" in fe.flags:
+                                print("    Unfuzzied female entry")
+                                fe.flags.remove("fuzzy")
+                                fe.previous_msgid = None
+                elif e.msgstr != female_value:
+                    print("INFO: new female translation detected:")
+                    print("  ORIG:   {}".format(e.msgid))
+                    print("  MALE:   {}".format(e.msgstr))
+                    print("  FEMALE: {}".format(female_value))
+                    fe = polib.POEntry(msgid=e.msgid, msgstr=female_value, msgctxt=CONTEXT_FEMALE)
+                    new_female_enties.append(fe)
 
             # translation is the same
             if e.msgstr == value and e.msgctxt == context:
+                print("  translation is the same for {}".format(e.msgid))
+                if "fuzzy" in e.flags:
+                    print("  {}  is fuzzy. Keeping fuzzy flag.".format(e.msgid))
                 continue
 
             # translation is the same as source
@@ -603,6 +586,10 @@ def file2msgstr(
                 e.flags.remove("fuzzy")
                 e.previous_msgid = None
 
+    # add newly found female entries
+    for nfe in new_female_enties:
+        po.append(nfe)
+
     return po
 
 
@@ -687,6 +674,7 @@ class TRANSEntry:
         self.context = None
         self.female = None
         self.comment = None
+        self.occurence = None
 
 
 class TRANSFile:
@@ -703,6 +691,7 @@ class TRANSFile:
         self.fformat = FILE_FORMAT[fext]
         self.pattern = self.fformat["pattern"]
         self.dotall = self.fformat["dotall"]
+        self.filepath = filepath
 
         try:  # comment for all entries in file
             self.comment = self.fformat["comment"]
@@ -723,10 +712,12 @@ class TRANSFile:
                     self.lines_female = self.load_lines(female_file)
                 else:
                     print("didn't find")
-            if self.lines_female and self.lines_female == self.lines:
-                print("  female lines are identical")
-            else:
-                print("  female lines are different")
+
+            if self.lines_female is not None:
+                if self.lines_female == self.lines:
+                    print("  female lines are identical")
+                else:
+                    print("  female lines are different")
 
         # protection again duplicate indexes, part 1
         seen = []
@@ -736,6 +727,7 @@ class TRANSFile:
 
             # index and value
             index = line[self.fformat["index"]]
+            entry.occurence = (filepath, str(index))
             entry.value = str(line[self.fformat["value"]])
 
             # skip invalid '000' entries in MSG files
@@ -785,7 +777,7 @@ class TRANSFile:
                     sys.exit(1)
 
             # sfall female extraction
-            if not is_source and self.lines_female and not (self.lines_female == self.lines):
+            if not is_source and (self.lines_female is not None) and not (self.lines_female == self.lines):
                 try:
                     female_line = [fl for fl in self.lines_female if fl[self.fformat["index"]] == entry.index][0]
                     entry.female = str(female_line[self.fformat["value"]])
@@ -845,13 +837,20 @@ def language_slug(po_filename):
     return slug
 
 
-def restore_female_entries(po: polib.POFile):
+def update_female_entries(po: polib.POFile):
     """
-    Unobsoletes and if necessary (un)fuzzies female strings that have a corresponding male counterpart.
+    (Un)obsoletes and if necessary (un)fuzzies female strings that have a corresponding male counterpart.
     (Male = no context)
     """
-    male_entries = {x.msgid: x for x in po if not x.previous_msgid and (x.msgctxt != CONTEXT_FEMALE)}
-    fuzzy_male_entries = {x.previous_msgid: x for x in po if x.previous_msgid and (x.msgctxt != CONTEXT_FEMALE)}
+    # this also includes fuzzies
+    male_entries = {x.msgid: x for x in po if ((x.msgctxt != CONTEXT_FEMALE) and (not x.obsolete))}
+    # and this is for matching male strings that were changed
+    fuzzy_male_entries = {
+        x.previous_msgid: x
+        for x in po
+        if ((x.previous_msgid is not None) and (x.msgctxt != CONTEXT_FEMALE) and (not x.obsolete))
+    }
+
     for e in po.obsolete_entries():
         if e.msgctxt != CONTEXT_FEMALE:
             continue
@@ -864,15 +863,23 @@ def restore_female_entries(po: polib.POFile):
             e.flags = male_entry.flags
             e.obsolete = False
 
-        # if a fuzzy male string was found, fixing female to have the same attributes
-        if (e.msgid not in male_entries) and (e.msgid in fuzzy_male_entries):
+        # else, check if a fuzzy male match exists, and fix female to have the same attributes
+        elif e.msgid in fuzzy_male_entries:
             male_entry = fuzzy_male_entries[e.msgid]
             e.msgid = male_entry.msgid
             e.previous_msgid = male_entry.previous_msgid
             if "fuzzy" not in e.flags:
                 e.flags.append("fuzzy")
             e.obsolete = False
-    return po
+
+    # and delete female entries with no non-obsolete male match
+    new_entries = [x for x in po if not (x.obsolete and (x.msgctxt == CONTEXT_FEMALE))]
+    meta = po.metadata
+    po2 = polib.POFile()
+    po2.metadata = meta
+    for e in new_entries:
+        po2.append(e)
+    return po2
 
 
 def unfuzzy_exact_matches(po: polib.POFile):
@@ -887,10 +894,11 @@ def unfuzzy_exact_matches(po: polib.POFile):
     This function unfuzzies such entries.
     """
     for e in po.fuzzy_entries():
-        if e.previous_msgid == e.msgid:
-            print("    Unfuzzied entry {}, msgid is the same as previous_msgid".format(e.occurrences))
+        if (e.previous_msgid == e.msgid) and (e.previous_msgctxt == e.msgctxt):
+            print("    Unfuzzied entry {}, exact match with previous".format(e.occurrences))
             e.flags.remove("fuzzy")
             e.previous_msgid = None
+            e.previous_msgctxt = None
     return po
 
 
